@@ -164,13 +164,16 @@ class BaseHandler(tornado.web.RequestHandler):
                 self.logger.warning(self.request.body)
 
         for key, value in self.request.arguments.items():
-            value = list(filter(None, map(lambda x: x.decode().strip(), value)))
+            value = list(filter(None, map(lambda x: x.decode('utf8', 'ignore').strip(), value)))
             if value:
                 kwargs[key] = value[0] if len(value) == 1 else value
 
         for key in ['page', 'size', 'order']:
             if kwargs.get(key) is not None:
-                kwargs[key] = int(kwargs[key])
+                try:
+                    kwargs[key] = int(kwargs[key])
+                except:
+                    pass
 
         self.args = Dict(kwargs)
         return Dict(kwargs)
