@@ -172,19 +172,20 @@ class SignupHandler(BaseHandler):
                 'token': token,
                 'created_at': datetime.datetime.now().replace(microsecond=0)
             })
-            ret = await awaitable(self.db.users.find_one_and_update({'username': doc['username']},
+            user = await awaitable(self.db.users.find_one_and_update({'username': doc['username']},
                                                                     {'$set': doc},
                                                                     upsert=True,
                                                                     return_document=True))
             update = {}
-            if not ret.id:
+            if not user.id:
                 update['id'] = await awaitable(self.db.users.seq_id)
                 if update['id'] == 1:
                     update['admin'] = True
             if update:
-                await awaitable(self.db.users.update_one({'_id': ret._id}, {'$set': update}))
+                await awaitable(self.db.users.update_one({'_id': user._id}, {'$set': update}))
+                user.update(update)
             self.set_cookie('token', token, expires_days=30)
-            self.finish({'err': 0, 'token': token})
+            self.finish({'err': 0, 'user': user})
         else:
             self.finish({'err': 1, 'msg': '信息未填写完整'})
 
@@ -207,7 +208,7 @@ class SigninHandler(BaseHandler):
                 query = {'username': username, 'password': self.encrypt(password)}
             user = await awaitable(self.db.users.find_one(query))
             if user:
-                ret = {'err': 0, 'token': user.token}
+                ret = {'err': 0, 'user': user}
                 if remember == 'on':
                     self.set_cookie('token', user.token, expires_days=30)
                 else:
@@ -223,10 +224,8 @@ class SigninHandler(BaseHandler):
 class UserHandler(BaseHandler):
 
     async def get(self):
-        user = self.current_user
-        if user:
-            user = dict([(k, v) for k, v in user.items() if k not in ['token', 'password', '_id']])
-            self.finish(user)
+        if self.current_user:
+            self.finish(self.current_user)
         else:
             self.finish({'err': 1, 'msg': '用户未登录'})
 
