@@ -29,7 +29,6 @@ from .http_utils import Response
 from .ip2region import Ip2Region
 from .log_utils import Logger
 from .log_utils import WatchedFileHandler
-from .processor import Processor
 from .rabbitmq import AioPika
 from .rabbitmq import Pika
 from .utils import AioQueue
@@ -58,8 +57,6 @@ from .utils import yaml_load
 
 warnings.filterwarnings("ignore")
 
-__version__ = '1.3.6'
-
 __all__ = [
     'awaitable', 'floor', 'ceil', 'to_str', 'to_bytes', 'tqdm', 'yaml_load', 'yaml_dump', 'pprint',
     'timeit', 'retry', 'aioretry', 'smart_decorator', 'synchronize', 'cached_property',
@@ -70,5 +67,5 @@ __all__ = [
     'Config', 'Logger', 'WatchedFileHandler',
     'Mongo', 'MongoClient', 'Redis', 'AioRedis', 'Motor', 'MotorClient',
     'Mysql', 'AioMysql', 'Pika', 'AioPika',
-    'Request', 'Response', 'Chrome', 'Crypt', 'Processor'
+    'Request', 'Response', 'Chrome', 'Crypt'
 ]
