@@ -35,10 +35,10 @@ from urllib.parse import urlencode
 from urllib.parse import urlparse
 from urllib.parse import urlunparse
 
+import aiohttp
 import chardet
 import pycurl
 import requests
-import tornado.curl_httpclient
 import yaml
 from bs4 import BeautifulSoup
 from lxml import etree
@@ -103,10 +103,10 @@ class ChunkedMultipartEncoderMonitor(MultipartEncoderMonitor):
 
     def __iter__(self):
         for i in range(self.chunks):
-            string = self.encoder.read(self.chunk_size)
-            self.bytes_read += len(string)
+            data = self.encoder.read(self.chunk_size)
+            self.bytes_read += len(data)
             self.callback(self)
-            yield string
+            yield data
 
 
 class Response:
@@ -119,17 +119,16 @@ class Response:
                 setattr(self, key, getattr(args[0], key))
         else:
             kwargs.setdefault('body', b'')
-            for key in ['code', 'reason', 'headers', 'body', 'request',
-                        'url', 'time', 'cookies', 'encoding']:
-                if key in kwargs:
-                    setattr(self, key, kwargs[key])
+            for k, v in kwargs.items():
+                setattr(self, k, v)
 
         if hasattr(self, 'headers') and isinstance(self.headers, httputil.HTTPHeaders):
             self.cookies = {}
             if self.code == 200:
+                sc = SimpleCookie()
                 for cookie in self.headers.get_list('Set-Cookie'):
-                    sc = SimpleCookie(cookie)
-                    self.cookies.update(dict(map(lambda x: (x[0], x[1].value), sc.items())))
+                    sc.load(cookie)
+                self.cookies = dict(map(lambda x: (x[0], x[1].value), sc.items()))
             self.headers = CaseInsensitiveDict(self.headers.items())
         elif not hasattr(self, 'headers'):
             self.headers = CaseInsensitiveDict()
@@ -261,7 +260,7 @@ class BaseRequest(metaclass=RequestMeta):
         if self.cookie:
             root = Path(self.cookie).parent
             if not root.exists():
-                os.makedirs(root)
+                root.mkdir(parents=True, exist_ok=True)
 
     def add_headers(self, headers):
         self.headers.update(headers)
@@ -355,7 +354,7 @@ class BaseRequest(metaclass=RequestMeta):
         try:
             return self._request(url, **kwargs)
         except Exception as e:
-            self.logger.error(f'url: {url}, retry: {retry}, exception: {e}')
+            self.logger.exception(f'request failed, url: {url}, retry: {retry}, exception: {e}')
             if retry <= 0:
                 return Response(url=url, code=599, reason=str(e))
             else:
@@ -445,6 +444,47 @@ class BaseRequest(metaclass=RequestMeta):
         return body
 
 
+class AsyncRequest(BaseRequest):
+
+    async def _retry(self, url, kwargs, retry):
+        _kwargs = copy.deepcopy(kwargs)
+        try:
+            return await self._request(url, **kwargs)
+        except Exception as e:
+            self.logger.exception(f'url: {url}, retry: {retry}, exception: {e}')
+            if retry <= 0:
+                return Response(url=url, code=599, reason=str(e))
+            else:
+                await asyncio.sleep(self.sleep)
+                return await self._retry(url, _kwargs, retry - 1)
+
+    async def download(self, url, filename=None, shards=1, **kwargs):
+        filename = Path(filename or Path(url).name)
+        filename.parent.mkdir(parents=True, exist_ok=True)
+        if shards > 1:
+            url, kwargs = self._prepare(url, kwargs)
+            kwargs['headers']['Range'] = 'bytes=0-0'
+            resp = await self.request(url, **kwargs)
+            filesize = int(resp.headers['Content-Range'].split('/')[-1])
+            block = int(math.ceil(filesize / shards))
+            futures = []
+            for i in range(shards):
+                start = block * i
+                end = min(block * (i + 1), filesize) - 1
+                _kwargs = copy.deepcopy(kwargs)
+                _kwargs['headers']['Range'] = f'bytes={start}-{end}'
+                futures.append(self.request(url, **_kwargs))
+            resps = await asyncio.gather(*futures)
+            with open(filename, 'wb') as fp:
+                for resp in resps:
+                    if resp.code == 206:
+                        fp.write(resp.body)
+        else:
+            resp = await self.request(url, **kwargs)
+            if resp.code == 200:
+                filename.write_bytes(resp.body)
+
+
 class Requests(BaseRequest):
 
     def __init__(self, **kwargs):
@@ -509,7 +549,7 @@ class Requests(BaseRequest):
         kwargs = copy.copy(kwargs)
         kwargs['headers']['Range'] = f'bytes={start}-{end}'
         kwargs['stream'] = True
-        method = kwargs.pop('method', 'GET')
+        method = kwargs.pop('method', 'GET').upper()
         resp = self.session.request(method, url, **kwargs)
         pbar = tqdm(total=end - start + 1, unit='B', unit_scale=True, desc=f'{start}')
         for chunk in resp.iter_content(chunk_size=4096):
@@ -523,8 +563,16 @@ class Requests(BaseRequest):
 
     def _request(self, url, **kwargs):
         url, kwargs = self._prepare(url, kwargs)
-        request_cookies = requests.utils.dict_from_cookiejar(self.session.cookies)
-        request_cookies.update(kwargs['cookies'])
+
+        cookies = requests.utils.dict_from_cookiejar(self.session.cookies)
+        cookies.update(kwargs['cookies'])
+        request = Dict(
+            url=url,
+            headers=kwargs['headers'],
+            data=kwargs['data'],
+            cookies=cookies,
+        )
+
         method = kwargs.pop('method', 'GET')
         resp = self.session.request(method, url, **kwargs)
         if kwargs.get('stream'):
@@ -539,21 +587,13 @@ class Requests(BaseRequest):
         self._finish(kwargs)
         self.save_cookie()
 
-        kwargs['method'] = method
-        response_cookies = requests.utils.dict_from_cookiejar(resp.cookies)
-        request = Dict(
-            url=url,
-            headers=kwargs['headers'],
-            data=kwargs['data'],
-            cookies=request_cookies
-        )
         response = Response(
             url=resp.url,
             encoding=resp.encoding,
             headers=resp.headers,
             body=content,
             code=resp.status_code,
-            cookies=response_cookies,
+            cookies=requests.utils.dict_from_cookiejar(resp.cookies),
             reason=resp.reason,
             request=request,
             time=resp.elapsed.microseconds / 1e6,
@@ -561,6 +601,75 @@ class Requests(BaseRequest):
         return response
 
 
+class Aiohttp(AsyncRequest):
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        cookiejar = aiohttp.CookieJar(unsafe=True)
+        self.session = aiohttp.ClientSession(cookie_jar=cookiejar)
+        self.load_cookie()
+
+    def __del__(self):
+        loop = asyncio.get_event_loop()
+        if loop.is_running():
+            loop.create_task(self.session.close())
+        else:
+            loop.run_until_complete(self.session.close())
+
+    def save_cookie(self):
+        if self.cookie:
+            self.session._cookie_jar.save(self.cookie)
+
+    def load_cookie(self):
+        if self.cookie and Path(self.cookie).exists():
+            self.session._cookie_jar.load(self.cookie)
+
+    def _prepare(self, url, kwargs):
+        url, kwargs = super()._prepare(url, kwargs)
+        kwargs.setdefault('verify_ssl', False)
+        kwargs.setdefault('raise_for_status', True)
+        kwargs.pop('progress', None)
+        kwargs.pop('chunked', None)
+        files = kwargs.pop('files')
+        if files:
+            # data = aiohttp.FormData()
+            # for k, v in kwargs['data']:
+            #     data.add_field(k, v)
+            # for k, v in files:
+            #     data.add_field(k, v[1], filename=v[0], content_type=v[2])
+            headers, data = kwargs.pop('headers'), kwargs.pop('data')
+            data = self.multipart(headers, data, files)
+            kwargs['data'] = data
+            kwargs['headers'] = headers
+        return url, kwargs
+
+    async def _request(self, url, **kwargs):
+        url, kwargs = self._prepare(url, kwargs)
+        cookies = self.session._cookie_jar.filter_cookies(url)
+        cookies = dict(map(lambda x: (x[0], x[1].value), cookies.items()))
+        cookies.update(kwargs['cookies'])
+        request = Dict(
+            url=url,
+            headers=kwargs['headers'],
+            data=kwargs['data'],
+            cookies=cookies,
+        )
+
+        method = kwargs.pop('method', 'GET').upper()
+        resp = await self.session.request(method, url, **kwargs)
+        self.save_cookie()
+        response = Response(
+            url=resp.url,
+            headers=resp.headers,
+            body=await resp.read(),
+            code=resp.status,
+            reason=resp.reason,
+            cookies=dict(map(lambda x: (x[0], x[1].value), resp.cookies.items())),
+            request=request,
+        )
+        return response
+
+
 class Pycurl(BaseRequest):
 
     def __init__(self, **kwargs):
@@ -585,6 +694,7 @@ class Pycurl(BaseRequest):
                 self.cookiejar.load(ignore_discard=True, ignore_expires=True)
         else:
             curl.setopt(pycurl.COOKIEFILE, '')
+            curl.setopt(pycurl.COOKIEJAR, '')
 
     def save_cookie(self, resp):
         req = MockRequest(resp.request)
@@ -618,7 +728,6 @@ class Pycurl(BaseRequest):
         # curl.setopt(pycurl.AUTOREFERER, 1)
         # 导致未知的403错误 https://video.wanmeikk.me/hls/a02689e5-d0f6-4a55-a590-75119b142a21/d6MJXL_kK9bOc1cTqAs3vVOWDA6wepIKOln9hG_oIT2vMXt81bvAsSaNmOVnaXAcM2BsEd87CSN9VlFIrLZuSw==.ts
         curl.setopt(pycurl.URL, urllib.parse.quote(url, safe='#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~'))
-
         curl.setopt(pycurl.TRANSFER_ENCODING, int(kwargs.get('chunked', False)))
         curl.setopt(pycurl.FOLLOWLOCATION, kwargs['allow_redirects'])
         curl.setopt(pycurl.CONNECTTIMEOUT, kwargs['timeout'])
@@ -739,8 +848,15 @@ class Pycurl(BaseRequest):
     def _request(self, url, **kwargs):
         url, kwargs = self._prepare(url, kwargs)
         self._curl_setup(self.c, url, **kwargs)
-        request_cookies = requests.utils.dict_from_cookiejar(self.cookiejar)
-        request_cookies.update(kwargs['cookies'])
+        cookies = requests.utils.dict_from_cookiejar(self.cookiejar)
+        cookies.update(kwargs['cookies'])
+        request = Dict(
+            url=url,
+            headers=kwargs.get('headers'),
+            data=kwargs.get('data'),
+            cookies=cookies,
+        )
+
         self.c.perform()
         self._finish(kwargs)
 
@@ -755,12 +871,6 @@ class Pycurl(BaseRequest):
         self.h.seek(0)
         self.h.truncate()
 
-        request = Dict(
-            url=url,
-            headers=kwargs.get('headers'),
-            data=kwargs.get('data'),
-            cookies=request_cookies,
-        )
         resp = Response(
             url=url,
             code=code,
@@ -773,7 +883,7 @@ class Pycurl(BaseRequest):
         return resp
 
 
-class TornadoClient(Pycurl):
+class TornadoClient(Pycurl, AsyncRequest):
 
     def __init__(self, max_clients=10, **kwargs):
         ''' This cannot handle 301/302 redirection cookies
@@ -781,18 +891,6 @@ class TornadoClient(Pycurl):
         super().__init__(**kwargs)
         self.http = CurlAsyncHTTPClient(max_clients=max_clients)
 
-    async def _retry(self, url, kwargs, retry):
-        _kwargs = copy.deepcopy(kwargs)
-        try:
-            return await self._request(url, **kwargs)
-        except tornado.curl_httpclient.CurlError as e:
-            self.logger.error(f'url: {url}, retry: {retry}, exception: {e}')
-            if retry <= 0:
-                return Response(url=url, code=e.code, reason=e.message)
-            else:
-                await asyncio.sleep(self.sleep)
-                return await self._retry(url, _kwargs, retry - 1)
-
     async def _request(self, url, **kwargs):
         url, kwargs = self._prepare(url, kwargs)
         curl_callback = partial(self._curl_setup, url=url, **kwargs)
@@ -802,33 +900,6 @@ class TornadoClient(Pycurl):
         self.save_cookie(resp)
         return resp
 
-    async def download(self, url, filename=None, shards=1, **kwargs):
-        filename = Path(filename or Path(url).name)
-        if shards > 1:
-            url, kwargs = self._prepare(url, kwargs)
-            kwargs['headers']['Range'] = 'bytes=0-0'
-            resp = await self.request(url, **kwargs)
-            filesize = int(resp.headers['Content-Range'].split('/')[-1])
-            block = int(math.ceil(filesize / shards))
-            futures = []
-            for i in range(shards):
-                start = block * i
-                end = min(block * (i + 1), filesize) - 1
-                _kwargs = copy.deepcopy(kwargs)
-                _kwargs['headers']['Range'] = f'bytes={start}-{end}'
-                futures.append(self.request(url, **_kwargs))
-            resps = await asyncio.gather(*futures)
-            filename.parent.mkdir(parents=True, exist_ok=True)
-            with open(filename, 'wb') as fp:
-                for resp in resps:
-                    if resp.code == 206:
-                        fp.write(resp.body)
-        else:
-            resp = await self.request(url, **kwargs)
-            if resp.code == 200:
-                filename.parent.mkdir(parents=True, exist_ok=True)
-                filename.write_bytes(resp.body)
-
 
 class Fetcher(BaseRequest):
 
@@ -962,9 +1033,10 @@ class Chrome(webdriver.Chrome):
 
 def Request(lib='requests', **kwargs):
     module = {
-        'pycurl': Pycurl,
         'requests': Requests,
-        'fetcher': Fetcher,
+        'aiohttp': Aiohttp,
+        'pycurl': Pycurl,
         'tornado': TornadoClient,
+        'fetcher': Fetcher,
     }
     return module[lib](**kwargs)
