@@ -122,21 +122,20 @@ class Dict(dict):
         for key, value in self.items():
             self.__setitem__(key, value)
 
-    def to_dict(self):
+    def dict(self):
         return DictUnwrapper(self)
 
     def __delattr__(self, key):
         try:
             del self[key]
-            return True
         except Exception:
-            return False
+            pass
 
     def __getattr__(self, key):
         try:
             return self[key]
         except Exception:
-            return None
+            pass
 
     def __setitem__(self, key, value):
         super().__setitem__(key, DictWrapper(value))
